'***********************************************************************
'
'    Filename        :     BCD-R-Dekade.Bas		
'    File Version    :     1.0
'    Datum           :     07.06.2008            	
'    Autor           :     Peter Verderber		
'    Funktion        :     BCD-Lastwiderstandsdekade	
'    Letzte Revision :     1.0
'    Datum           :     07.06.2008           
'    Ziel PIC        :     16F627  	
'    Compiler        :     iL_BAS16 - Professional  V5.7-10  23.11.2007 
'    
'***********************************************************************
'*****                Definitionen und Konstanten                  *****
'***********************************************************************

Date
Time

$NCALDEF

'$LIST BIN          'erzeugt zustzlich eine Binrdatei 
'$LIST OBJ2HEX      'das Hexfile bekommt anstatt OBJ die Dateiendung HEX   

define device 16f628a,wdt_off,hs_osc,cmcfg7,MCLR_INT,OSC2_IO,T0CS_INT              
  
xtal 4.0 

'$debug 3,ra,4
                      
tris ra,%000011
tris rb,%00000000          

output rb,%10000000

define load        = auto as word      'Last-Widerstand        
define relais      = auto as byte      'Ansteuerung Relais
define nn          = auto as word      'Hilfsvariablen
define rr          = auto as word 

define taste_down  = ra,0              'Taste DOWN
define taste_up    = ra,1              'Taste UP
define dp_on       = ra,2              'Dezimalpunkt ein
define displ_on    = ra,3              'Display ein
define displ_clear = ra,4              'Display nullen
define count       = rb,7              'Zhlimpuls

goto main                              'Start Hauptprogramm

'***********************************************************************
'*****                   Unterprogramme                            *****
'***********************************************************************

init:                        'Variablen & Display initalisieren
  
  set displ_on               'Display einschalten
  set dp_on                  'Dezimalpunkt einschalten
   
  output rb,%10000000        'Alle Relais aus R = 63 Ohm
 
  gosub counter_clear        
  
  for nn = 1 to 88           'Segment-Test Anzeige 8.8
    gosub counter_count
  next nn
  
  for nn = 1 to 3            'Display 3 mal blinken
    set displ_on
    wait 250  
    res displ_on
    wait 250  
  next nn
  
  wait 1000
  
  gosub counter_clear        'Zhler nullen
  
  for nn = 1 to 63           'Anzeige Startwert R = 63 Ohm 
    gosub counter_count
  next nn        

  res dp_on                  'Dezimalpunkt ausschalten
  
  let load = 630             'Last-Widerstand = 63 Ohm
  let nn   = 0
  
  return
     
'-----------------------------------------------------------------------

counter_clear:               'Zhler nullen                           

  set ra,4       
  pulsout ra,4,10,L 
  return       

'-----------------------------------------------------------------------

counter_count:               'Zhlimpuls generieren              
  
  res rb,7       
  pulsout rb,7,10,H       
  return  

$tron

'----------------------------------------------------------------------- 

set_relais:                  'Relais einschalten

  let relais = relais xor %111111
  output rb,relais
  res rb,7 
  return	
 
'----------------------------------------------------------------------- 
  
show_load:                   'Anzeige Lastwiderstand in Ohm
  let relais = 0       
  let rr     = load / 10     'Umwandeln in eine 2-stellige Zahl
  
  gosub counter_clear        'Zhler nullen 
  
  for nn = 1 to rr           'Anzeigebereich 10 bis 63 Ohm)
    gosub counter_count
    inc relais               'Relais setzen von 1 bis 63
  next nn
 
  if rr < 10 then gosub klein_R else res dp_on
  
  return
  
  
  klein_R:
    gosub counter_clear      'Zhler nullen
    set dp_on                'Dezimalpunkt einschalten bei R < 10 Ohm
    let rr = rr * 10         'Umwandel in 2-stellige Zahl
       
    for nn = 1 to rr         'Anzeigebereich 1.0 bis 9.0 Ohm)
      gosub counter_count
    next nn
  return  
  
'-----------------------------------------------------------------------  

load_up_down:                'Tastenabfrage UP / DOWN

  if taste_up = 0 then wait 50         'Taste UP
  if taste_up = 0 then gosub inc_load
  
  if taste_down = 0 then wait 50       ' Taste DOWN
  if taste_down = 0 then gosub dec_load
   
  return
    
   inc_load:                           'Widerderstand inkrementieren
     if load < 630 then let load = load + 10
     gosub show_load
     gosub set_relais
     wait 100
     return     
    
   dec_load:                           'Widerstand dekrementieren
     if load > 10  then let load = load - 10
     gosub show_load
     gosub set_relais
     wait 100
     return   
    
  
'***********************************************************************
'*****                     Hauptprogramm                           *****
'***********************************************************************

main:
  gosub init                 'Variablen initialisieren

  loop:                      'Tastenabfrage UP / DOWN
    gosub load_up_down
    wait 100
    goto loop
  
goto main       

end                          'Ende gut, alles gut       
        
       